import time

from netforce.model import Model, fields, get_model
from netforce.utils import get_file_path
from . import utils

class ImportProduct(Model):
    _name="clinic.import.product"
    _transient=True

    _fields={
        'date': fields.DateTime("Date"),
        'file': fields.File("File"),
        'result': fields.Text("Success"),
        'hcode': fields.Char("Hospital Code"),
    }
    
    def get_hcode(self,context={}):
        settings=get_model("settings").browse(1)
        hcode=settings.hospital_code or ""
        return hcode

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
        'hcode': get_hcode,
    }

    def import_product(self,ids,context={}):
        obj=self.browse(ids)[0]
        fname=obj.file
        fpath=get_file_path(fname)
        lines=utils.read_excel(fpath,show_datetime=True)
        if not lines:
            raise Exception("Wrong File")
        for line in lines:
            hcode=line.get('hcode18','0')
            if not hcode:
                hcode='0'
            hcode=int(hcode)
            hcode=str(hcode)
            if obj.hcode==hcode:
                prod_code=line.get("code31")
                prod_name=line.get("eponame")
                prod_ids=get_model("clinic.product").search([['code','=',prod_code]])
                if not prod_ids:
                    vals={
                        'code': prod_code,
                        'name': prod_name,
                        'uom_id': 2, #XXX Unit
                        'type': 'stock',
                    }
                    prod_id=get_model('product').create(vals)
                    print("create product ", prod_id)

ImportProduct.register()
