import time
from datetime import datetime, timedelta

from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

from . import utils

class ReportStaffFeeSum(Model):
    _name="clinic.report.staff.fee.sum"
    _string="Report Staff Fee Summary"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        "staff_id": fields.Many2One("clinic.staff","Staff"),
        "type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),
        'date_from': _get_date_from,
        'date_to': _get_date_to,
        'type': 'nurse',
    }
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model('company').browse(company_id)
        year, month=time.strftime("%Y-%m").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        time_start='%s-%s-01'%(year,str(month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(month).zfill(2),total_day)

        defaults=context.get('defaults')
        if defaults:
            year,month,day=defaults['date'].split("-")
            time_start='%s-%s-01'%(year,str(month).zfill(2))
            time_stop='%s-%s-%s'%(year,str(month).zfill(2),total_day)
        staff_id=None
        staff_type="nurse"
        if ids:
            obj=self.browse(ids)[0]
            staff_type=obj.type
            staff_id=obj.staff_id.id
            month=obj.date_from.split("-")[1]
            time_start=obj.date_from
            time_stop=obj.date_to
        
        # generate list of date
        fmt='%Y-%m-%d'
        start=datetime.strptime(time_start,fmt)
        stop=datetime.strptime(time_stop,fmt)
        days=(stop-start).days
        day_list=[]
        for day in range(days+1):
            dt=start+timedelta(days=day)
            day_list.append(dt.strftime(fmt))

        cols=[]
        for date in day_list:
            cols.append({'name': date[8:10]})

        lines=[]
        total_all={}
        total_sum=0.0
        dom=[]
        dom.append(['type','=',staff_type])
        for staff in get_model("clinic.staff").search_browse(dom):
            if staff_id:
                if staff_id!=staff.id:
                    continue
            line={
                'staff_id': staff.id,
                'staff_name': '%s'%(staff.name),
                'sub_lines': [],
            }
            total_amt=0.0
            for date in day_list:
                if not total_all.get(date):
                    total_all[date]=0.0
                dom=[]
                dom.append(['date','=',date])
                dom.append(['staff_id','=',staff.id])
                amt=0
                results=get_model("clinic.labor.cost.line").search_read(dom,['amount'])
                if results:
                    for result in results:
                        amt+=result['amount'] or 0.0
                line['sub_lines'].append({
                    'amt': amt,
                    'last': False,
                })
                total_all[date]+=amt # all day
                total_amt+=amt
            # total of each nurse
            line['sub_lines'].append({
                'amt': round(total_amt,2),
                'last': True,
            })
            lines.append(line)
            total_sum+=total_amt

        total_lines=[]
        for date in day_list:
            total_lines.append({
                'amt': total_all.get(date,0),
            })
        total_lines.append({
            'amt': round(total_sum,2),
        })

        month_str=utils.MONTHS['th_TH'][int(month)]
        start=int(time_start[8:10])
        stop=int(time_stop[8:10])
        diff=stop-start
        data={
            'company_name': company.name or "",
            'parent_company_name': company.parent_id.name or "",
            'cols': cols,
            'lines': lines,
            'total_lines': total_lines,
            'month': month_str,
            'from': time_start,
            'to': time_stop,
            'is_duration': diff+1 < total_day,
            'year': year,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

ReportStaffFeeSum.register()
